﻿using WingServerCommon.Service;
using System.Threading.Tasks;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Request.User;
using WingInterfaceLibrary.Interface;
using WingInterfaceLibrary.DTO.Package;
using System;
using WingServerCommon.Interfaces.Cache;
using WingServerCommon.Log;

namespace WingUserModule.Service;

public partial class UserService : JsonRpcService, IUserService
{
    /// <summary>
    /// 根据唯一标识查询软件安装包
    /// </summary>
    /// <param name="request"></param>
    /// <returns></returns>
    public async Task<PackagesDTO> GetPackageByUniquedCodeAsync(GetPackageByUniquedCodeRequest request)
    {
        var uniquedCode = request.UniquedCode;
        var ipAddress = request.IpAddress;
        var dto = await _packageDBService.GetPackageByUniquedCodeDBAsync(new GetPackageByUniquedCodeDBRequest
        {
            UniquedCode = uniquedCode,
        });
        if (dto != null && !string.IsNullOrWhiteSpace(dto.UploadUserCode))
        {
            var userInfo = await _userDBService.FindUserByCodeAsync(dto.UploadUserCode);
            if (userInfo != null)
            {
                dto.UploadUserName = userInfo.DisplayName;
            }
        }
        var isOverseas = false;
        if (!string.IsNullOrWhiteSpace(ipAddress) && dto != null && !string.IsNullOrWhiteSpace(dto.Code) && !string.IsNullOrWhiteSpace(dto.FileUrl))
        {
            var ipManager = CacheMaintenance.Instance.Get<IIPManager>();
            if (ipManager != null)
            {
                var clientIPInfo = ipManager.GetIPInfo(ipAddress);
                if (clientIPInfo != null && !clientIPInfo.IsChinaMainland)
                {
                    isOverseas = true;
                    dto.FileUrl = dto.FileUrl.Replace("cdn-bj.fis.plus", "cdn-hk.fis.plus").Replace("flyinsono-bj-1300984704.cos.ap-beijing.myqcloud.com", "flyinsono-hk-1300984704.cos.ap-hongkong.myqcloud.com");
                }
            }
        }
        Logger.WriteLineInfo($"UserService GetPackageByUniquedCodeAsync, package name:{dto.Name}, version:{dto.Version}, ip:{ipAddress}, isOverseas:{isOverseas}, fileUrl:{dto.FileUrl}");
        return dto;
    }

    /// <summary>
    /// 新增/更新软件安装包
    /// </summary>
    /// <param name="request"></param>
    /// <returns></returns>
    public async Task<string> AddOrUpdatePackageAsync(AddOrUpdatePackageRequest request)
    {
        var package = request.PackageInfo;
        if (string.IsNullOrWhiteSpace(package.UploadUserCode))
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            package.UploadUserCode = userCode;
        }
        if (package.CreateTime < new DateTime(2000, 1, 1))
        {
            package.CreateTime = DateTime.UtcNow;
        }
        package.UpdateTime = DateTime.UtcNow;
        var code = await _packageDBService.AddOrUpdatePackageDBAsync(package);
        return code;
    }
}
