﻿using System.Threading.Tasks;
using System;
using WingInterfaceLibrary.Interface;
using WingUserModule.Entity;
using System.Collections.Generic;
using System.Linq;
using WingInterfaceLibrary.Request.User;
using RemoveUsersFromOrganizationRequest = WingInterfaceLibrary.Request.User.RemoveUsersFromOrganizationRequest;
using SetUserOrganizationInfoRequest = WingInterfaceLibrary.Request.User.SetUserOrganizationInfoRequest;
using WingInterfaceLibrary.DTO.User;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.Request;
using System.Collections.Concurrent;
using WingServerCommon.Service;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Interface.DBInterface;
using WingServerCommon.Mapper;
using WingUserModule.Processor;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.DTO.Organization;
using WingInterfaceLibrary.Result.UserFeature;
using WingInterfaceLibrary.Request.DBRequest;
using WingUserModule.Helper;
using WingInterfaceLibrary.DTO.Rank;
using WingInterfaceLibrary.Request.Authentication;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using WingUserModule.Common;
using System.Text.RegularExpressions;
using WingServerCommon.Log;
using WingServerCommon.Interfaces.Cache;
using WingServerCommon.Interfaces.PasswordVerify;
using WingServerCommon.Interfaces.Login;
using WingInterfaceLibrary.Notifications;
using WingInterfaceLibrary.DTO.Schedule;
using WingInterfaceLibrary.DTO.GradingProtection;
using WingInterfaceLibrary.DTO.Message;
using WingInterfaceLibrary.Internal.Interface;
using WingInterfaceLibrary.Internal.Request;
using WingInterfaceLibrary.LiveConsultation;
using WingInterfaceLibrary.Enum.NotificationEnum;
using System.Net.Http;
using System.Linq;
using Newtonsoft.Json.Linq;
using System.IO;
using WingInterfaceLibrary.DTO.Management;
using System.Threading;
using WingServerCommon.Utilities.Executors;
using WingInterfaceLibrary.ResearchEdition;

namespace WingUserModule.Service
{
    public partial class UserService : JsonRpcService, IUserService
    {
        private IAdminDBService _adminDBService;
        private IUserDBService _userDBService;
        private IAuthenticationService _authenticationService;
        private IIdentityApplyDBService _identityApplyDBService;
        private IStorageDBService _storageDBService;
        private IStorageService _storageService;
        private IOrganizationDBService _organizationDBService;
        private IDeviceInfoDBService _deviceInfoDBService;
        private IRoleDBService _roleDBService;
        private IRankDBService _rankDBService;
        private ISMSService _smsService;
        private IEmailService _emailService;
        private IEducationService _educationService;
        private IPatientInfoDBService _patientInfoDBService;
        private IUserFeaturesDBService _userFeaturesDBService;
        private IUserAuthorityGroupsDBService _userAuthorityGroupsDBService;
        private ConcurrentDictionary<string, VerifyCodeInfo> _emailVerifyCodes = new ConcurrentDictionary<string, VerifyCodeInfo>();

        private readonly SequenceExecutor<string> _userDatasExecutor = new SequenceExecutor<string>("userDatasUpdate");


        private IRecordsInfoDBService _recordsInfoDBService;

        private IStatisticDBService _statisticDBService;
        private ICMSTemplateDBService _cmsTemplateDBService;
        private IMessageDBService _messageDBService;
        private IAnnouncementDBService _announcementDBService;
        private int _heartRateSeconds;
        private int _defaultQueryDays => EnvironmentConfigManager.GetParammeter<IntParameter>("LiveConsultation", "DefaultQueryDays").Value;
        private UserHeartRateManager _userHeartRateManager;

        private UserLoginLock _userLoginLock;

        private ILiveConsultationService _liveConsultationService;
        private ScanLoginManager _scanLoginManager;
        private string _RegexPhone = "^1(3[0-9]|4[01456879]|5[0-35-9]|6[2567]|7[0-8]|8[0-9]|9[0-35-9])\\d{8}$";
        private string _RegexPhoneOther = @"^\d{4,15}$";

        private string _RegexEmail = "^[A-Za-z0-9-._]+@[A-Za-z0-9-]+(.[A-Za-z0-9]+)?(.[A-Za-z]{2,6})$";

        private string _RegexUserName = @"^[\u4e00-\u9fffA-Za-z0-9]{2,20}$";
        private INotificationService _notificationService;
        private UserPasswordExpireVerify _passwordExpireVerify;
        private int _keepPwdTimes;
        private string _webSocketUrl = string.Empty;
        private int _liveConsultationRateSeconds => EnvironmentConfigManager.GetParammeter<IntParameter>("LiveConsultation", "HeartRateSeconds").Value;
        private string _cmsUrl => EnvironmentConfigManager.GetParammeter<StringParameter>("CMS", "CMSUrl").Value;
        private string _manageUrl => EnvironmentConfigManager.GetParammeter<StringParameter>("CMS", "ManageUrl").Value;
        private int _remoteControlHeartRateSeconds => EnvironmentConfigManager.GetParammeter<IntParameter>("Device", "RemoteControlHeartRateSeconds").Value;
        private int _courseVisitorHeartRateSeconds => EnvironmentConfigManager.GetParammeter<IntParameter>("Course", "HeartRateSeconds").Value;
        private int _leaveRoomTimes => EnvironmentConfigManager.GetParammeter<IntParameter>("LiveConsultation", "LeaveRoomTimes").Value;
        private string _courseUrl => EnvironmentConfigManager.GetParammeter<StringParameter>("Course", "MyCourseUrl").Value;
        private string _courseHomeUrl => EnvironmentConfigManager.GetParammeter<StringParameter>("Course", "CourseHomeUrl").Value;
        private string _courseAPIUrl => EnvironmentConfigManager.GetParammeter<StringParameter>("Course", "CourseAPIUrl").Value;
        private string _curCDNHost => EnvironmentConfigManager.GetParammeter<StringParameter>("Storage", "CDNServerUrl").Value;
        private readonly string _agentServer = EnvironmentConfigManager.GetParammeter<StringParameter>("User", "ThirdPartyAgentServer").Value;
        private readonly string _agentToken = EnvironmentConfigManager.GetParammeter<StringParameter>("User", "ThirdPartyAgentToken").Value;
        private bool _isResearchEditionService => EnvironmentConfigManager.GetParammeter<BoolParameter>("General", "IsResearchEditionService").Value;
        private string _updateMode = string.Empty;
        private string _grayscaleRules = string.Empty;
        private string _pcFISVersion = string.Empty;
        private string _pcFlyinsonoVersion = string.Empty;
        private string _pcInstallPackageVersion = string.Empty;
        private string _androidVersion = string.Empty;
        private string _sonoPostVersion = string.Empty;
        private string _windowsFISLibVersion = string.Empty;
        private string _description = string.Empty;
        private string _storageServer = string.Empty;
        private string _thirdPartyServer = string.Empty;
        private readonly HttpClient _httpClient = new HttpClient();
        private IMasterInteractionCenterService _masterInteractionCenterService;
        private int _scanLoginValidSeconds;

        private IChatMessageDBService _chatMessageDBService;
        private IRtcService _rtcService;
        private ICourseDBService _courseDBService;
        private IUserGroupDBService _userGroupDBService;
        private IPackagesDBService _packageDBService;
        private IResearchEditionService _researchEditionService;

        /// <summary>
        /// Init service
        /// </summary>
        public override void Load(JsonRpcClientPool jsonRpcClientPool)
        {
            base.Load(jsonRpcClientPool);
            _adminDBService = GetProxy<IAdminDBService>();
            _authenticationService = GetProxy<IAuthenticationService>();
            _storageDBService = GetProxy<IStorageDBService>();
            _storageService = GetProxy<IStorageService>();
            _organizationDBService = GetProxy<IOrganizationDBService>();
            _deviceInfoDBService = GetProxy<IDeviceInfoDBService>();
            _roleDBService = GetProxy<IRoleDBService>();
            _rankDBService = GetProxy<IRankDBService>();
            _userDBService = GetProxy<IUserDBService>();
            _positionDBService = GetProxy<IPositionDBService>();
            _identityApplyDBService = GetProxy<IIdentityApplyDBService>();
            _smsService = GetProxy<ISMSService>();
            _emailService = GetProxy<IEmailService>();
            _patientInfoDBService = GetProxy<IPatientInfoDBService>();
            _userFeaturesDBService = GetProxy<IUserFeaturesDBService>();
            _userAuthorityGroupsDBService = GetProxy<IUserAuthorityGroupsDBService>();
            _recordsInfoDBService = GetProxy<IRecordsInfoDBService>();
            _statisticDBService = GetProxy<IStatisticDBService>();
            _cmsTemplateDBService = GetProxy<ICMSTemplateDBService>();
            _messageDBService = GetProxy<IMessageDBService>();
            _announcementDBService = GetProxy<IAnnouncementDBService>();
            _liveConsultationService = GetProxy<ILiveConsultationService>();
            _chatMessageDBService = GetProxy<IChatMessageDBService>();
            _rtcService = GetProxy<IRtcService>();
            _courseDBService = GetProxy<ICourseDBService>();
            _userGroupDBService = GetProxy<IUserGroupDBService>();
            _educationService = GetProxy<IEducationService>();
            _packageDBService = GetProxy<IPackagesDBService>();
            _researchEditionService = GetProxy<IResearchEditionService>();
            LoginProcessorCreator.InitProcessorCreator(_userDBService, _authenticationService, checkedLoginIsLocked);
            _heartRateSeconds = ConfigurationManager.GetParammeter<IntParameter>("User", "HeartRateSeconds").Value;
            CacheMaintenance.Instance.Get<IUserInfoManager>().RegisterDBFunction(LoadUserModule, LoaUserModules);
            CacheMaintenance.Instance.Get<IGradingProtectionConfigManager>().RegisterDBFunction(LoadGradingProtectionConfigModule, LoadGradingProtectionConfigModules);
            var loginLockTimes = ConfigurationManager.GetParammeter<IntParameter>("Authentication", "LoginLockTimes").Value;
            _userLoginLock = new UserLoginLock(loginLockTimes);
            _userLoginLock.LoginLockInfoChanged += LoginLockInfoChanged;
            _userHeartRateManager = new UserHeartRateManager(SetOnlineState);
            _notificationService = GetProxy<INotificationService>();
            _keepPwdTimes = ConfigurationManager.GetParammeter<IntParameter>("Authentication", "KeepPwdTimes").Value;
            _passwordExpireVerify = new UserPasswordExpireVerify(new Func<IList<string>, IList<PasswordVerifyModel>>(LoadUserDataByCodes), true);
            _passwordExpireVerify.SendNoticeEvent += OnSendNoticeEvent;
            _webSocketUrl = ConfigurationManager.GetParammeter<StringParameter>("Notification", "WebSocketUrl").Value;
            _storageServer = ConfigurationManager.GetParammeter<StringParameter>("Storage", "StorageServer").Value;
            _thirdPartyServer = ConfigurationManager.GetParammeter<StringParameter>("User", "ThirdPartyServer").Value;
            _scanLoginValidSeconds = ConfigurationManager.GetParammeter<IntParameter>("Authentication", "ScanLoginValidSeconds").Value;
            _httpClient.Timeout = TimeSpan.FromMinutes(10);
            _masterInteractionCenterService = GetProxy<IMasterInteractionCenterService>();
            _scanLoginManager = new ScanLoginManager(_masterInteractionCenterService);
            ReloadUpgradeConfig().GetAwaiter().GetResult();
        }

        public override void LoadAfterRegister()
        {
            try
            {
                WatchingUpgrade();
                RegisterStatistic();
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"UserService LoadAfterRegister err, {ex}");
            }
        }

        public async Task<bool> ReloadUpgradeConfig()
        {
            CommonConfigManager.AddParamters("Upgrade", Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Upgrade", "UpgradeSetting.json"));
            _updateMode = CommonConfigManager.GetParammeter<StringParameter>("Upgrade", "Upgrade", "UpdateMode").Value;
            _grayscaleRules = CommonConfigManager.GetParammeter<StringParameter>("Upgrade", "Upgrade", "GrayscaleRules").Value;
            _pcFISVersion = CommonConfigManager.GetParammeter<StringParameter>("Upgrade", "Upgrade", "PCFISVersion").Value;
            _pcFlyinsonoVersion = CommonConfigManager.GetParammeter<StringParameter>("Upgrade", "Upgrade", "PCFlyinsonoVersion").Value;
            _pcInstallPackageVersion = CommonConfigManager.GetParammeter<StringParameter>("Upgrade", "Upgrade", "PCInstallPackageVersion").Value;
            _androidVersion = CommonConfigManager.GetParammeter<StringParameter>("Upgrade", "Upgrade", "AndroidVersion").Value;
            _sonoPostVersion = CommonConfigManager.GetParammeter<StringParameter>("Upgrade", "Upgrade", "SonoPostVersion").Value;
            _windowsFISLibVersion = CommonConfigManager.GetParammeter<StringParameter>("Upgrade", "Upgrade", "WindowsFISLibVersion").Value;
            _description = CommonConfigManager.GetParammeter<StringParameter>("Upgrade", "Upgrade", "Description").Value;
            return true;
        }

        /// <summary>
        /// 发送通知委托         
        /// </summary>
        /// <param name="request">用户登录请求实体</param>
        /// <returns></returns>
        private void OnSendNoticeEvent(Object sender, SendNoticeArgs args)
        {
            if (args != null && args.VerifyList != null && args.VerifyList.Count > 0)
            {
                foreach (var item in args.VerifyList)
                {
                    if (item.SurplusTime == 0)
                    {
                        //正常，不需要预警
                        continue;
                    }
                    //发送通知
                    var userPwdNotice = new PasswordExpiredWarningNotification()
                    {
                        SurplusTime = item.SurplusTime,
                    };
                    //Logger.WriteLineInfo($"Notify PasswordExpiredWarningNotification user {item.ClientId}");
                    var sendNotificationRequest = new SendNotificationRequest()
                    {
                        ClientId = item.ClientId,
                        Message = userPwdNotice,
                        JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(userPwdNotice),
                        NotificationType = userPwdNotice.NotificationType,
                        TransactionType = TransactionTypeEnum.Session,
                        RelevanceCode = item.ClientId,
                        ReceiverType = ApplicantTypeEnum.Client
                    };
                    var result = _notificationService.PostMessageAsync(sendNotificationRequest).GetAwaiter().GetResult();
                    if (result)
                    {
                        Logger.WriteLineInfo($"Send update user password notice success, user {item.ClientId}");
                    }
                    else
                    {
                        Logger.WriteLineInfo($"Send update user password notice fail, user {item.ClientId}");
                    }
                }
            }
        }

        /// <summary>
        /// 登录验证密码是否过期
        /// </summary>
        /// <param name="token">token字符串</param>
        /// <returns></returns>
        private async Task<bool> VerifyUserPasswordExpire(string token)
        {
            var passwordOverdue = GetGradingProtectionConfigCache("PasswordOverdueLimit01");
            if (!passwordOverdue)
            {
                return false;
            }
            var userCode = await GetClientIdByTokenAsync(token);
            var userInfo = await _userDBService.FindUserPasswordByCodeAsync(userCode);
            if (userInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotExisted, "User not found");
                return false;
            }
            //验证密码是否过期
            bool passwordExpired = false;
            var verifyReq = new PasswordVerifyModel()
            {
                ClientId = userInfo.UserCode,
                PasswordLastUpdateTime = userInfo.PasswordUpdateTime
            };
            var verifyResult = _passwordExpireVerify.VerifyPasswordExpire(verifyReq);
            if (verifyResult.SurplusTime >= 0)
            {
                //表示未过期，不做处理
            }
            else
            {
                //表示已过期
                passwordExpired = true;
            }
            return passwordExpired;
        }

        /// <summary>
        /// 加载User密码数据信息
        /// </summary>
        /// <param name="request">用户codes列表</param>
        /// <returns></returns>
        private IList<PasswordVerifyModel> LoadUserDataByCodes(IList<string> codes)
        {
            IList<PasswordVerifyModel> resultList = new List<PasswordVerifyModel>();
            if (codes?.Count > 0)
            {
                var userPwdList = _userDBService.FindUsersByUserCodesAsync(codes.ToList()).GetAwaiter().GetResult();
                if (userPwdList?.Count > 0)
                {
                    resultList = userPwdList.FindAll(c => !string.IsNullOrEmpty(c.SecretPassword)).Select(d => new PasswordVerifyModel()
                    {
                        ClientId = d.UserCode,
                        PasswordLastUpdateTime = d.PasswordUpdateTime
                    }).ToList();
                }
            }
            return resultList;
        }

        private WingServerCommon.Interfaces.Cache.CacheUserDTO LoadUserModule(string code)
        {
            return LoaUserModules(new List<string> { code }).FirstOrDefault();
        }

        private List<WingServerCommon.Interfaces.Cache.CacheUserDTO> LoaUserModules(IList<string> codes)
        {
            List<UserDTO> dtos = _userDBService.FindUserListByUserCodesAsync(codes.ToList()).Result;
            return dtos.MappingTo<List<WingServerCommon.Interfaces.Cache.CacheUserDTO>>();
        }

        private WingServerCommon.Interfaces.Cache.CacheGradingProtectionConfigDTO LoadGradingProtectionConfigModule(string code)
        {
            return LoadGradingProtectionConfigModules(new List<string> { code }).FirstOrDefault();
        }

        private List<WingServerCommon.Interfaces.Cache.CacheGradingProtectionConfigDTO> LoadGradingProtectionConfigModules(IList<string> codes)
        {
            List<GradingProtectionConfigDTO> dtos = _adminDBService.FindGradingProtectionConfigListAsync(codes.ToList()).Result;
            var cacheList = new List<WingServerCommon.Interfaces.Cache.CacheGradingProtectionConfigDTO>();
            foreach (var dto in dtos)
            {
                cacheList.Add(new CacheGradingProtectionConfigDTO
                {
                    Code = dto.Code,
                    Name = dto.Name,
                    ConfigType = dto.ConfigType,
                    Value = dto.Value
                });
            }
            return cacheList;
        }

        private async void LoginLockInfoChanged(object sender, LoginLockEventArgs args)
        {
            try
            {
                if (args != null)
                {
                    var passwordError = GetGradingProtectionConfigCache("PasswordErrorLimit01");
                    if (!passwordError)
                    {
                        return;
                    }
                    var result = await _userDBService.UpdateUserLoginLockInfoAsync(
                      args.Code,
                      new LoginLockInfoDTO()
                      {
                          LoginDate = args.LoginDate,
                          Times = args.Times
                      });
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"UserService LoginLockInfoChanged err, {ex}");
            }

        }
        private void ThrowCustomerException(CustomerRpcCode customerRpcCodeEnum, string msg)
        {
            ThrowRpcException((int)customerRpcCodeEnum, msg);
        }

        /// <summary>
        /// 手机号验证方法
        /// </summary>
        /// <param name="value"></param>
        public void VerifyPhoneValue(string value, string languageCode)
        {
            if (languageCode == "zh-CN" || languageCode == null)
            {
                if (!string.IsNullOrWhiteSpace(value) && !Regex.IsMatch(value, _RegexPhone))
                {
                    ThrowCustomerException(CustomerRpcCode.PhoneIncorrect, "Phone incorrect!");
                }
            }
            else 
            {
                if (!string.IsNullOrWhiteSpace(value) && !Regex.IsMatch(value, _RegexPhoneOther))
                {
                    ThrowCustomerException(CustomerRpcCode.PhoneIncorrect, "Phone incorrect!");
                }
            }
        }
        /// <summary>
        /// Email验证方法
        /// </summary>
        /// <param name="value"></param>
        public void VerifyEmailValue(string value)
        {
            if (!string.IsNullOrWhiteSpace(value) && !Regex.IsMatch(value, _RegexEmail))
            {
                ThrowCustomerException(CustomerRpcCode.EmailIncorrect, "Email incorrect!");
            }
        }

        /// <summary>
        /// 账号名称验证方法
        /// </summary>
        /// <param name="value"></param>
        public void VerifyAccountValue(string value)
        {
            if (string.IsNullOrWhiteSpace(value))
            {
                ThrowCustomerException(CustomerRpcCode.AccountIsEmpty, "Account is empty");
            }
            else if (value.Length < 1 || value.Length > 50)
            {
                ThrowCustomerException(CustomerRpcCode.AccountLengthError, "Account length incorrect");
            }
        }

        /// <summary>
        /// 验证 验证码
        /// </summary>
        /// <param name="value"></param>
        public void VerifyVerificationCodeValue(string value)
        {
            if (!string.IsNullOrWhiteSpace(value) && value.Length < 4 || value.Length > 6)
            {
                ThrowCustomerException(CustomerRpcCode.VerificationCodeIllegality, "VerificationCode is illegality");
            }
        }

        /// <summary>
        /// 客户端心跳
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> HeartRateAsync(TokenRequest request)
        {
            _userHeartRateManager.AddOrUpdate(request.Token);
            var result = SetOnlineState(request.Token, true);
            return result;
        }

        private bool SetOnlineState(string token, bool isOnline)
        {
            return _authenticationService.SetOnlineStateAsync(new SetOnlineStateRequest { Token = token, IsOnline = isOnline }).Result;
        }

        /// <summary>
        /// 获取用户信息
        /// </summary>
        /// <param name="sessionId">授权Id</param>
        /// <returns>是否成功</returns>
        public async Task<UserDTO> GetUserInfoAsync(GetUserInfoRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var dbRequest = new FindUserRequest();
            dbRequest.Code = userCode;
            var userDTO = await _userDBService.FindUserAsync(dbRequest);
            if (userDTO == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotExisted, "User not found");
            }
            return userDTO;
            // var userInfo = userDTO.MappingTo<UserDTO>();
            // if (!string.IsNullOrEmpty(userInfo.DepartmentCode))
            // {
            //     var orgInfo = CacheMaintenance.Instance.Get<IOrganizationsManager>().Get(userInfo.DepartmentCode);
            //     if (orgInfo != null)
            //     {
            //         userInfo.DepartmentName = orgInfo.OrganizationName;
            //         userInfo.DepartmentShortCode = orgInfo.ShortCode;
            //     }
            // }
            // if (!string.IsNullOrEmpty(userInfo.RootOrganizationCode))
            // {
            //     var orgInfo = CacheMaintenance.Instance.Get<IOrganizationsManager>().Get(userInfo.RootOrganizationCode);
            //     if (orgInfo != null)
            //     {
            //         userInfo.RootOrganizationName = orgInfo.OrganizationName;
            //         userInfo.RootOrganizationShortCode = orgInfo.ShortCode;
            //     }
            // }
            // return userInfo;
        }

        /// <summary>
        /// 修改用户信息
        /// </summary>
        /// <param name="sessionId">授权Id</param>
        /// <param name="userInfo">用户信息</param>
        /// <returns>是否成功</returns>
        public async Task<bool> AlterUserInfoAsync(AlterUserInfoRequest request)
        {
            try
            {
                var userCode = await GetClientIdByTokenAsync(request.Token);
                var findUserDBRequest = new FindUserRequest();
                findUserDBRequest.Code = userCode;
                var srcUserInfoDTO = await _userDBService.FindUserAsync(findUserDBRequest);
                var isNeesUpdate = false;
                var org = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = request.OrganizationCode, ShortCode = request.ShortCode });
                if (org == null)
                {
                    if (!string.IsNullOrEmpty(request.FullName) || !string.IsNullOrEmpty(request.Email) || !string.IsNullOrEmpty(request.Phone) || !string.IsNullOrEmpty(request.NickName))
                    {
                        //只修改用户信息                        
                        if (!string.IsNullOrEmpty(request.NickName))
                        {
                            VerifyAccountValue(request.NickName);
                        }
                        srcUserInfoDTO.NickName = request.NickName;
                        if (!string.IsNullOrEmpty(request.FullName))
                        {
                            VerifyAccountValue(request.FullName);
                        }
                        srcUserInfoDTO.FullName = request.FullName;
                        if (!string.IsNullOrWhiteSpace(request.Phone))
                        {
                            //解密验证手机号是否合法
                            VerifyPhoneValue(request.Phone, request.LanguageCode);
                            var accountState = await CheckLoginTypeWithExcept(new CheckLoginTypeRequest { AnyAccount = request.Phone.ToString() }, new List<string> { userCode });
                            if (accountState != (int)LoginProcessorTypeEnum.Unregistered)
                            {
                                ThrowCustomerException(CustomerRpcCode.PhoneBinded, "Phone repeat binding");
                            }
                        }
                        srcUserInfoDTO.Phone = request.Phone;
                        if (!string.IsNullOrWhiteSpace(request.Email))
                        {
                            VerifyEmailValue(request.Email.ToString());
                            var accountState = await CheckLoginTypeWithExcept(new CheckLoginTypeRequest { AnyAccount = request.Email.ToString() }, new List<string> { userCode });
                            if (accountState != (int)LoginProcessorTypeEnum.Unregistered)
                            {
                                ThrowCustomerException(CustomerRpcCode.EmailBinded, "EMail repeat binding");
                            }
                        }
                        if (srcUserInfoDTO.IsDirector && !string.IsNullOrWhiteSpace(srcUserInfoDTO.RootOrganizationCode))
                        {
                            var organization = await _organizationDBService.GetOrganizationDetailAsync
                            (
                                new GetOrganizationDBRequest
                                {
                                    OrganizationCode = srcUserInfoDTO.RootOrganizationCode
                                }
                            );
                            if (organization != null && organization.Directors != null && organization.Directors.Exists(c => c == userCode))
                            {
                                organization.Directors.Remove(userCode);
                                //清除绑定的机构负责人
                                await _organizationDBService.SaveOrganizationAsync(organization.OrganizationCode, organization, true);
                            }
                        }
                        srcUserInfoDTO.Email = request.Email;
                        srcUserInfoDTO.UpdateTime = DateTime.UtcNow;
                        if (request.OrganizationCode != srcUserInfoDTO?.OrganizationCode)
                        {
                            isNeesUpdate = true;
                        }
                        srcUserInfoDTO.OrganizationCode = "";
                        var res = await _userDBService.UpdateUserAsync(srcUserInfoDTO.UserCode, srcUserInfoDTO.MappingTo<UserPasswordDTO>(), request.ExtensionData, true);
                        if (res && isNeesUpdate)
                        {
                            res = await _userDBService.ClearPersonDefaultTemplateDBAsync(userCode, request.LanguageCode);
                        }
                        return res;
                    }
                }
                if (!string.IsNullOrEmpty(org?.OrganizationCode) && string.IsNullOrEmpty(request.OrganizationCode))
                {
                    request.OrganizationCode = org.OrganizationCode;
                }
                if (request.OrganizationCode != srcUserInfoDTO?.OrganizationCode)
                {
                    isNeesUpdate = true;
                }
                var allDevice = await _deviceInfoDBService.FindDeviceInfoListByOrganizationCodeAsync(request.OrganizationCode);
                //如果是部门
                if (org?.RootCode != org?.OrganizationCode)
                {
                    var rootDevices = await _deviceInfoDBService.FindDeviceInfoListByOrganizationCodeAsync(org?.RootCode);
                    allDevice.AddRange(rootDevices);
                }
                var sharedDevices = allDevice.Where(e => e.IsAutoShared == true).Select(f => f.DeviceCode).Distinct().ToList();
                if (srcUserInfoDTO.BindDevices != null && srcUserInfoDTO.BindDevices.Count > 0)
                {
                    //sharedDevices = sharedDevices.Except(srcUserInfoDTO.BindDevices).ToList();
                    sharedDevices = sharedDevices.Concat(srcUserInfoDTO.BindDevices).Distinct().ToList();
                }
                string oldUrl = string.Empty;
                if (!string.IsNullOrWhiteSpace(request.Phone))
                {
                    //解密验证手机号是否合法
                    VerifyPhoneValue(request.Phone, request.LanguageCode);
                    var accountState = await CheckLoginTypeWithExcept(new CheckLoginTypeRequest { AnyAccount = request.Phone.ToString() }, new List<string> { userCode });
                    if (accountState != (int)LoginProcessorTypeEnum.Unregistered)
                    {
                        ThrowCustomerException(CustomerRpcCode.PhoneBinded, "Phone repeat binding");
                    }
                }
                srcUserInfoDTO.Phone = request.Phone;
                if (!string.IsNullOrWhiteSpace(request.Email))
                {
                    VerifyEmailValue(request.Email.ToString());
                    var accountState = await CheckLoginTypeWithExcept(new CheckLoginTypeRequest { AnyAccount = request.Email.ToString() }, new List<string> { userCode });
                    if (accountState != (int)LoginProcessorTypeEnum.Unregistered)
                    {
                        ThrowCustomerException(CustomerRpcCode.EmailBinded, "EMail repeat binding");
                    }
                }
                srcUserInfoDTO.Email = request.Email;
                // delegate get organization method
                var getOrganizationDTO = async (string code) => string.IsNullOrWhiteSpace(code)
                ? null
                : await _organizationDBService.GetOrganizationDetailAsync
                (
                    new GetOrganizationDBRequest
                    {
                        OrganizationCode = code
                    }
                );
                OrganizationDTO newOrgModel = await getOrganizationDTO(request.OrganizationCode);
                OrganizationDTO oldOrgModel = await getOrganizationDTO(srcUserInfoDTO.OrganizationCode);
                if (newOrgModel?.RootCode != oldOrgModel?.RootCode)//换机构
                {
                    //清除角色、岗位身份、机构负责人、病人创建人、病人操作人、角色申请记录等业务字段,不清除绑定的设备，添加新的设备
                    CleanBindInfo(srcUserInfoDTO, sharedDevices);
                    //重新选择医院，清理自己绑定的助理和急诊专家信息
                    srcUserInfoDTO.BindAssistantDoctorUserCode = "";
                    srcUserInfoDTO.BindAssistantUserCode = "";
                    srcUserInfoDTO.BindEmergencyDeviceCode = "";
                    srcUserInfoDTO.BindEmergencyExpertCode = "";
                    //更换机构，如果是机构负责人，则移除此负责人
                    if (!string.IsNullOrEmpty(oldOrgModel?.OrganizationCode))
                    {
                        if (oldOrgModel.Directors != null && oldOrgModel.Directors.Exists(c => c == userCode))
                        {
                            oldOrgModel.Directors.Remove(userCode);
                            //清除绑定的机构负责人
                            await _organizationDBService.SaveOrganizationAsync(oldOrgModel.OrganizationCode, oldOrgModel, true);
                        }
                    }
                    //清理其他用户绑定我为助理和专家的信息
                    var otherUserList = await _userDBService.GetUserListByBindUserCodeAsync(userCode);
                    if (otherUserList?.Count > 0)
                    {
                        foreach (var item in otherUserList)
                        {
                            if (item.BindAssistantDoctorUserCode == userCode)
                            {
                                item.BindAssistantDoctorUserCode = "";
                            }
                            if (item.BindAssistantUserCode == userCode)
                            {
                                item.BindAssistantUserCode = "";
                            }
                            if (item.BindEmergencyExpertCode == userCode)
                            {
                                item.BindEmergencyExpertCode = "";
                            }
                            await _userDBService.UpdateUserAsync(item.UserCode, item.MappingTo<UserPasswordDTO>(), string.Empty, true);
                        }
                    }
                }
                else
                {
                    srcUserInfoDTO.BindDevices.AddRange(sharedDevices);
                    //换部门
                    if (newOrgModel?.OrganizationCode != oldOrgModel?.OrganizationCode)
                    {
                        srcUserInfoDTO.PositionCodes = new List<string>();
                    }
                }
                srcUserInfoDTO.NickName = request.NickName;
                if (!string.IsNullOrEmpty(request.NickName))
                {
                    VerifyAccountValue(request.NickName);
                }
                srcUserInfoDTO.FullName = request.FullName;
                srcUserInfoDTO.OrganizationCode = request.OrganizationCode;
                if (!string.IsNullOrEmpty(request.OrganizationCode))
                {
                    VerifyAccountValue(request.OrganizationCode);
                }
                srcUserInfoDTO.UpdateTime = DateTime.UtcNow;
                if (!string.IsNullOrEmpty(request.HeadImageUrl))
                {
                    if (!Regex.IsMatch(request.HeadImageUrl, @"[a-zA-z]+://[^\s]*"))
                    {
                        ThrowCustomerException(CustomerRpcCode.HeadImageUrlNotValid, "HeadImageUrl is not valid");
                    }
                    srcUserInfoDTO.HeadImageUrl = request.HeadImageUrl.ToUrlToken();
                }
                var result = await _userDBService.UpdateUserAsync(srcUserInfoDTO.UserCode, srcUserInfoDTO.MappingTo<UserPasswordDTO>(), request.ExtensionData, true);
                if (result && isNeesUpdate)
                {
                    result = await _userDBService.ClearPersonDefaultTemplateDBAsync(userCode, request.LanguageCode);
                }
                return result;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"user module alter user info failed, ex:{ex}");
                throw;
            }
        }

        /// <summary>
        /// 重置用户在机构的数据
        /// </summary>
        /// <param name="srcUserInfoDTO"></param>
        private void CleanBindInfo(UserDTO srcUserInfoDTO, List<string> sharedDevices)
        {
            //清除角色、设备分享关系、岗位身份、机构负责人、病人创建人、病人操作人、角色申请记录等业务字段
            //srcUserInfoDTO.RoleCodes = new List<string>();
            srcUserInfoDTO.BindDevices = sharedDevices;
            srcUserInfoDTO.AuthorityGroups = new List<string>();
            srcUserInfoDTO.RankCodes = new List<string>();
            srcUserInfoDTO.PositionCodes = new List<string>();
            srcUserInfoDTO.BindAssistantDoctorUserCode = "";
            srcUserInfoDTO.BindAssistantUserCode = "";
            //异步修改业务数据
            _userDatasExecutor.Add(OnUpdateDatas, srcUserInfoDTO.UserCode);
        }


        private bool OnUpdateDatas(string userCode)
        {
            try
            {
                //查到该用户作为机构负责人的机构
                var allOrganizationFilter = new Dictionary<string, string>();
                allOrganizationFilter.Add("Director", userCode);
                var allOrganizations = _organizationDBService.GetOrganizationPageAsync((new PageFilterRequest { CurrentPage = 1, PageSize = 999999, Filter = allOrganizationFilter, IsFuzzy = false }));
                var needOrgs = allOrganizations.Result.PageData;
                foreach (var orgInfo in needOrgs)
                {
                    orgInfo.Directors.Remove(userCode);
                    _organizationDBService.SaveOrganizationAsync(orgInfo.OrganizationCode, orgInfo);
                }
                var patientInfoList = _patientInfoDBService.GetPatientsByOperatorAsync(new GetPatientsByOperatorRequest { UserCode = userCode });
                if (patientInfoList.Result.Count > 0)
                {
                    foreach (var item in patientInfoList.Result)
                    {
                        item.CreatorCode = "";
                        item.UpdateUsers?.Remove(userCode);
                        _patientInfoDBService.UpdatePatientsAsync(item.PatientCode, item, null, true);
                    }
                }
                //检查记录的创建人
                var recordFilter = new Dictionary<string, string>();
                recordFilter.Add("CreatorCode", userCode);
                var recordList = _recordsInfoDBService.FindRecordInfoPagesAsync(new FindRecordInfoPagesRequest { CurrentPage = 1, PageSize = 999999, Filter = recordFilter, IsFuzzy = true });
                if (recordList.Result.DataCount > 0)
                {
                    foreach (var item in recordList.Result.PageData)
                    {
                        item.CreatorCode = "";
                        _recordsInfoDBService.UpdateRecordInfoByCodeAsync(item.RecordCode, item, null, true);
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"OnUpdateDatas failed,userCode:{userCode}, ex:{ex}");
            }
            return true;
        }

        /// <summary>
        /// 个人中心-修改头像昵称
        /// </summary>
        /// <param name="sessionId">授权Id</param>
        /// <param name="userInfo">用户信息</param>
        /// <returns>是否成功</returns>
        public async Task<bool> AlterPersonInfoAsync(AlterPersonInfoRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrEmpty(request.NickName) && string.IsNullOrEmpty(request.HeadImageUrl))
            {
                ThrowCustomerException(CustomerRpcCode.NickNameAndHeadImageUrlEmpty, "NickName and HeadImageUrl Empty error");
            }
            var srcUserInfoDTO = await _userDBService.FindUserByCodeAsync(userCode);
            if (!string.IsNullOrEmpty(request.NickName))
            {
                if (request.NickName.Length > 50)
                {
                    ThrowCustomerException(CustomerRpcCode.NicknameTooLong, "Nickname is too long");
                }
                srcUserInfoDTO.NickName = request.NickName;
            }
            if (!string.IsNullOrEmpty(request.HeadImageUrl))
            {
                if (!Regex.IsMatch(request.HeadImageUrl, @"[a-zA-z]+://[^\s]*"))
                {
                    ThrowCustomerException(CustomerRpcCode.HeadImageUrlNotValid, "HeadImageUrl is not valid");
                }
                srcUserInfoDTO.HeadImageUrl = request.HeadImageUrl.ToUrlToken();
            }
            srcUserInfoDTO.UpdateTime = DateTime.UtcNow;
            var result = await _userDBService.UpdateUserAsync(srcUserInfoDTO.UserCode, srcUserInfoDTO.MappingTo<UserPasswordDTO>(), string.Empty);
            return result;
        }

        /// <summary>
        /// 获取用户信息列表
        /// </summary>
        /// <param name="request">请求体</param>
        /// <returns>用户信息列表</returns>
        public async Task<List<UserExtendDTO>> GetUserListAsync(GetUserListRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var dbRequest = new GetUserListDBRequest
            {
                Keyword = request.Keyword,
                OrganizationCode = request.OrganizationCode,
                OrganizationQueryType = request.OrganizationQueryType,
                RankCode = request.RankCode,
                PositionCode = request.PositionCode,
                ExceptUsers = new List<string>(),
                RoleCodes = request.RoleCodes
            };
            if (request.ExceptSelf)
            {
                dbRequest.ExceptUsers.Add(userCode);
            }
            var users = await _userDBService.GetUserListAsync(dbRequest);
            var result = users.MappingTo<List<UserExtendDTO>>();
            var findAuthenticationRolesRequest = new FindAuthenticationRolesRequest { Language = request.Language };
            var roles = await FindAllRolesAsync(findAuthenticationRolesRequest);
            foreach (var item in result)
            {
                if (item.RoleCodes != null && item.RoleCodes.Count > 0)
                {
                    item.RoleName = roles.FirstOrDefault(x => x.RoleCode == item.RoleCodes[0])?.RoleName;
                }
                var itemRequest = new GetUserStatusRequest { UserCode = item.UserCode };
                item.UserStatus = await _liveConsultationService.GetUserStatusAsync(itemRequest);
            }
            return result;
        }

        /// <summary>
        /// 根据组织和岗位身份查询用户列表信息(分页)
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>用户信息</returns>
        /// <remarks>POST</remarks>
        public async Task<PageResult<UserExtendDTO>> GetUserPagesAsync(GetUserPageRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var dbRequest = new GetUserListPagesDBRequest
            {
                Keyword = request.Keyword,
                OrganizationCode = request.OrganizationCode,
                OrganizationQueryType = request.OrganizationQueryType,
                RankCode = request.RankCode,
                PositionCode = request.PositionCode,
                ExceptUsers = new List<string>(),
                RoleCodes = request.RoleCodes,
                PageIndex = request.PageIndex,
                PageSize = request.PageSize,
                IsAgent = request.IsAgent
            };
            if (request.ExceptSelf)
            {
                dbRequest.ExceptUsers.Add(userCode);
            }
            var users = await _userDBService.GetUserListPagesDBAsync(dbRequest);
            var result = new PageResult<UserExtendDTO>();
            result.PageIndex = request.PageIndex;
            result.PageSize = request.PageSize;
            result.TotalCount = users.TotalCount;
            if (users.PageData != null && users.PageData.Count > 0)
            {
                result.PageData = users.PageData.MappingTo<List<UserExtendDTO>>();
                var findAuthenticationRolesRequest = new FindAuthenticationRolesRequest { Language = request.Language };
                var roles = await FindAllRolesAsync(findAuthenticationRolesRequest);
                foreach (var item in result.PageData)
                {
                    if (item.RoleCodes != null && item.RoleCodes.Count > 0)
                    {
                        item.RoleName = roles.FirstOrDefault(x => x.RoleCode == item.RoleCodes[0])?.RoleName;
                    }
                    var itemRequest = new GetUserStatusRequest { UserCode = item.UserCode };
                    item.UserStatus = await _liveConsultationService.GetUserStatusAsync(itemRequest);
                }
            }
            return result;
        }

        /// <summary>
        /// 从当前用户组织中删除用户
        /// </summary>
        /// <param name="request">请求体</param>
        /// <returns>删除数量</returns>
        public async Task<bool> RemoveUsersFromOrganizationAsync(RemoveUsersFromOrganizationRequest request)
        {
            var users = await _userDBService.FindUserListByUserCodesAsync(request.UserCodes);
            foreach (var item in users)
            {
                item.OrganizationCode = "";
                CleanBindInfo(item, new List<string>());
                await _userDBService.UpdateUserAsync(item.UserCode, item.MappingTo<UserPasswordDTO>(), "", true);

            }
            return true;
        }

        /// <summary>
        /// 设置用户组织信息
        /// </summary>
        /// <param name="request">请求体</param>
        /// <returns></returns>
        public async Task<bool> SetUserOrganizationInfoAsync(SetUserOrganizationInfoRequest request)
        {
            var userInfo = await _userDBService.FindUserByCodeAsync(request.UserCode);
            if (!string.IsNullOrWhiteSpace(request.OrganizationCode))
            {
                var detailRequest = new GetOrganizationDBRequest { OrganizationCode = request.OrganizationCode };
                var newOrg = await _organizationDBService.GetOrganizationDetailAsync(detailRequest);
                if (newOrg.RootCode != userInfo.RootOrganizationCode)
                {
                    CleanBindInfo(userInfo, new List<string>());
                }
                userInfo.OrganizationCode = request.OrganizationCode;
            }
            if (request.PositionCodes != null && request.PositionCodes.Count > 0)
            {
                userInfo.PositionCodes = request.PositionCodes;
            }
            if (request.RankCodes != null && request.RankCodes.Count > 0)
            {
                userInfo.RankCodes = request.RankCodes;
            }
            if (request.RoleCodes != null && request.RoleCodes.Count > 0)
            {
                userInfo.RoleCodes = request.RoleCodes;
            }
            return await _userDBService.UpdateUserAsync(userInfo.UserCode, userInfo.MappingTo<UserPasswordDTO>(), string.Empty);
        }

        private async Task<string> AddFileTokenInfo(string url)
        {
            var result = await _storageDBService.StorageAsync(url);
            return result?.Token;
        }
        private async Task<string> GetFileUrlByToken(string token)
        {
            var result = await _storageDBService.GetStorageRouteAsync(token);
            return result?.StorageRoute;
        }

        public async Task<PageResult<ShareDeviceUserDTO>> GetShareDeviceUsersAsync(GetShareDeviceUsersPageRequest request)
        {
            var deviceCode = request.DeviceCode;
            var userCode = await GetClientIdByTokenAsync(request.Token);
            //检查设备编码不为空
            if (string.IsNullOrWhiteSpace(deviceCode))
            {
                ThrowCustomerException(CustomerRpcCode.DeviceCodeIsEmpty, "Required parameter:DeviceCode cannot be empty");
            }
            //查询设备对象
            var deviceDb = await _deviceInfoDBService.FindDeviceInfoByCodeAsync(deviceCode);
            if (deviceDb == null)
            {
                ThrowCustomerException(CustomerRpcCode.NoExistDevice, "Device does not exist");
            }
            var shareDeviceUsers = new List<ShareDeviceUserDTO>();
            var filter = new Dictionary<string, string>();
            filter.Add("DeviceCode", deviceCode);
            var userListResult = await _userDBService.FindUserPagesCustomAsync(new PageFilterRequest
            {
                CurrentPage = request.PageIndex,
                PageSize = request.PageSize,
                Filter = filter,
                IsFuzzy = true
            });
            if (userListResult.PageData.Count > 0)
            {
                var organizations = await _organizationDBService.FindOrganizationListByCodesAsync(userListResult.PageData.Select(u => u.OrganizationCode).ToList());
                var rootOrganizations = await _organizationDBService.FindOrganizationListByCodesAsync(organizations.Select(o => o.RootCode).ToList());
                var ranks = await GetUserRanks(userListResult.PageData);
                foreach (var item in userListResult.PageData)
                {
                    var rankNames = ranks.Where(v => item.RankCodes.Contains(v.RankCode)).Select(c => c.RankName).ToList();
                    var organization = organizations.FirstOrDefault(o => o.OrganizationCode == item.OrganizationCode);
                    var rootOrganization = rootOrganizations.FirstOrDefault(r => r.OrganizationCode == organization?.RootCode);
                    var shareDeviceUserDTO = new ShareDeviceUserDTO()
                    {
                        UserCode = item.UserCode,
                        FullName = item.FullName,
                        UserName = item.UserName,
                        Phone = item.Phone,
                        HeadImageUrl = item.HeadImageUrl,
                        RankNames = rankNames.Select(f => f.ToString()).ToList(),
                        RootOrganizationCode = rootOrganization?.RootCode,
                        RootOrganizationName = rootOrganization?.OrganizationName
                    };
                    shareDeviceUsers.Add(shareDeviceUserDTO);
                }
            }
            var result = new PageResult<ShareDeviceUserDTO>();
            result.PageIndex = request.PageIndex;
            result.PageSize = request.PageSize;
            result.TotalCount = userListResult.DataCount;
            result.PageData = shareDeviceUsers;
            return result;
        }

        /// <summary>
        /// 根据登录用户获取权限列表
        /// </summary>
        /// <param name="request">token</param>
        /// <returns>权限列表</returns>
        public async Task<List<UserFeatureInfoResult>> GetUserFeatureListAsync(TokenRequest request)
        {
            if (_isResearchEditionService)
            {
                var dataList = await _researchEditionService.GetResearchProjectUserFeatureListAsync(request);
                return dataList;
            }

            var userCode = await GetClientIdByTokenAsync(request.Token);
            var userInfo = await _userDBService.FindUserByCodeAsync(userCode);
            if (userInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotExisted, "User not found");
            }
            var result = new List<UserFeatureInfoResult>();
            var allFeaturesResult = await _userFeaturesDBService.FindUserFeaturePagesAsync(new PageFilterRequest { CurrentPage = 1, PageSize = 9999, Filter = new Dictionary<string, string>(), IsFuzzy = true });
            var allFeatures = allFeaturesResult.PageData.MappingTo<List<UserFeatureInfoResult>>();
            //todo针对 机构管理员，怎么处理
            if (!string.IsNullOrWhiteSpace(userInfo.OrganizationCode) && userInfo.IsDirector)
            {
                var deleteRecord = "DeleteRecord";
                if (userInfo.UserFeatureCodes == null || !userInfo.UserFeatureCodes.Any() || !userInfo.UserFeatureCodes.Contains(deleteRecord))
                {
                    allFeatures.RemoveAll(x => x.UniqueCode == deleteRecord);
                }
                var userGroups = "UserGroups";
                if (userInfo.UserFeatureCodes == null || !userInfo.UserFeatureCodes.Any() || !userInfo.UserFeatureCodes.Contains(userGroups))
                {
                    allFeatures.RemoveAll(x => x.UniqueCode == userGroups);
                }
                if (userInfo.UserFeatureCodes == null || userInfo.UserFeatureCodes.Count <= 0)
                {
                    //移除远程管理员权限
                    var item = allFeatures.Find(c => c.FeatureCode == "RemoteManager");
                    allFeatures.Remove(item);
                    item = allFeatures.Find(c => c.FeatureCode == "RemoteMaintain");
                    allFeatures.Remove(item);
                }
                else if (userInfo.UserFeatureCodes.Count > 0)
                {
                    var itemManager = allFeatures.Find(c => c.FeatureCode == "RemoteManager");
                    allFeatures.Remove(itemManager);
                    var itemMaintain = allFeatures.Find(c => c.FeatureCode == "RemoteMaintain");
                    allFeatures.Remove(itemMaintain);
                    if (userInfo.UserFeatureCodes.Exists(c => c == "RemoteManager"))
                    {
                        allFeatures.Add(itemManager);
                    }
                    else if (userInfo.UserFeatureCodes.Exists(c => c == "RemoteMaintain"))
                    {
                        allFeatures.Add(itemMaintain);
                    }
                }
                return allFeatures;
            }

            if (userInfo.RoleCodes != null && userInfo.RoleCodes.Count > 0)//当前用户的角色（目前只支持单个角色）
            {
                var roleDB = await _roleDBService.FindRoleByCodeAsync(userInfo.RoleCodes[0]);
                var userAuthorityGroupDB = await _userAuthorityGroupsDBService.FindUserAuthorityGroupByCodeAsync(roleDB?.UserGroupCode);
                result = allFeatures.Where(f => userAuthorityGroupDB.Features.Contains(f.FeatureCode)).ToList();
            }
            if (userInfo.UserFeatureCodes?.Count > 0)
            {
                var otherFeatureData = allFeatures.Where(f => userInfo.UserFeatureCodes.Contains(f.FeatureCode)).ToList();
                if (otherFeatureData?.Count > 0)
                {
                    result.AddRange(otherFeatureData);
                }
            }
            //没有权限则给普通用户的权限
            if (result.Count == 0)
            {
                var roleDB = await _roleDBService.FindRoleByCodeAsync("Role_GeneralUser");
                var userAuthorityGroupDB = await _userAuthorityGroupsDBService.FindUserAuthorityGroupByCodeAsync(roleDB?.UserGroupCode);
                result = allFeatures.Where(f => userAuthorityGroupDB.Features.Contains(f.FeatureCode)).ToList();
            }
            var resultList = result.MappingTo<List<UserFeatureInfoResult>>();
            //半选特殊处理
            foreach (var item in result)
            {
                GetUserFatherFeature(item, resultList, allFeatures);
            }
            if (userInfo.AssociatedInfos != null && userInfo.AssociatedInfos.Count > 0)
            {
                var associatedFeature = userInfo.AssociatedInfos.Select(s => new UserFeatureInfoResult
                {
                    FeatureCode = s.ETitle.Replace(" ", ""),
                    FeatureName = s.ETitle,
                    UniqueCode = s.ETitle.Replace(" ", "")
                });
                resultList.AddRange(associatedFeature);
            }
            //医院是兽用类型的屏蔽会诊
            if (!string.IsNullOrEmpty(userInfo.RootOrganizationCode))
            {
                var orgInfo = CacheMaintenance.Instance.Get<IOrganizationsManager>().Get(userInfo.RootOrganizationCode);
                if (orgInfo.PatientType == (int)OrganizationPatientTypeEnum.Animals)
                {
                    resultList = resultList.Where(x => x.FeatureCode != "Consultation").ToList();
                }
            }
            return resultList;
        }

        private void GetUserFatherFeature(UserFeatureInfoResult item, List<UserFeatureInfoResult> list, List<UserFeatureInfoResult> allFeatures)
        {
            if (string.IsNullOrWhiteSpace(item.FatherCode))
            {
                return;
            }
            var fatherItem = allFeatures.FirstOrDefault(x => x.FeatureCode == item.FatherCode);
            if (!list.Select(x => x.FeatureCode).Contains(fatherItem.FeatureCode))
            {
                list.Add(fatherItem);
                GetUserFatherFeature(fatherItem, list, allFeatures);
            }
        }


        /// <summary>
        /// 根据用户编码获取详情
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>用户信息</returns>
        public async Task<UserInfoByCodeDTO> GetUserByCodeAsync(GetUserByCodeRequest request)
        {
            var dbRequest = new FindUserRequest();
            dbRequest.Code = request.UserCode;
            var userDTO = await _userDBService.FindUserAsync(dbRequest);
            if (userDTO == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotExisted, "User not found");
            }
            var userInfo = userDTO.MappingTo<UserInfoByCodeDTO>();
            if (!string.IsNullOrWhiteSpace(userDTO.BindAssistantUserCode))
            {
                var assistant = await _userDBService.FindUserByCodeAsync(userDTO.BindAssistantUserCode);
                userInfo.BindAssistantUserName = assistant.FullName ?? assistant.UserName;
            }
            if (!string.IsNullOrWhiteSpace(userDTO.BindAssistantDoctorUserCode))
            {
                var assistantDoctor = await _userDBService.FindUserByCodeAsync(userDTO.BindAssistantDoctorUserCode);
                userInfo.BindAssistantDoctorUserName = assistantDoctor.FullName ?? assistantDoctor.UserName;
            }
            return userInfo;
        }

        public async Task<bool> IsDefaultPasswordAsync(TokenRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var userInfo = await _userDBService.FindUserPasswordByCodeAsync(userCode);
            if (userInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotExisted, "User not found");
            }
            if (string.IsNullOrWhiteSpace(userInfo.SecretPassword) || userInfo.SecretPassword == "Symmetry_mk9xah8N")//abc123
            {
                return true;
            }
            return false;
        }

        /// <summary>
        /// 获取用户职称
        /// </summary>
        /// <param name="userList"></param>
        /// <returns></returns>
        private async Task<List<RankDTO>> GetUserRanks(IEnumerable<UserDTO> userList)
        {
            var rankCodes = new List<string>();
            foreach (var item in userList)
            {
                if (item.RankCodes != null && item.RankCodes.Any())
                {
                    rankCodes.AddRange(item.RankCodes);
                }
            }
            if (rankCodes.Any())
            {
                return await _rankDBService.GetAllRanksAsync(new GetRanksDBRequest
                {
                    RankCodes = rankCodes
                });
            }
            return new List<RankDTO>();
        }

        private void checkedLoginIsLocked(string userCode)
        {
            var isLocked = _userLoginLock.GetLoginIsLocked(userCode);
            if (isLocked)
            {
                ThrowCustomerException(CustomerRpcCode.AccountIsLocked, "Account is locked");
            }
        }

        /// <summary>
        /// 更新用户设置
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>结果</returns>
        /// <value>true</value>
        /// <errorCodes>UserNotExisted</errorCodes>
        public async Task<bool> SetCommonSettingsAsync(CommonSettingsRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var userInfo = await _userDBService.FindUserPasswordByCodeAsync(userCode);
            if (userInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotExisted, "User not found");
            }
            var findUserDBRequest = new FindUserRequest();
            findUserDBRequest.Code = userCode;
            var srcUserInfoDTO = await _userDBService.FindUserAsync(findUserDBRequest);
            if (srcUserInfoDTO == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotFind, "User not found");
            }
            switch (request.SettingType)
            {
                case CommonSettingsEnum.Signature:
                    if (!string.IsNullOrEmpty(request.Value))
                    {
                        if (!Regex.IsMatch(request.Value, @"[a-zA-z]+://[^\s]*"))
                        {
                            ThrowCustomerException(CustomerRpcCode.SignatureUrlNotValid, "Signature is not valid");
                        }
                        srcUserInfoDTO.Signature = request.Value.ToUrlToken();
                    }
                    break;
                case CommonSettingsEnum.Language:
                    srcUserInfoDTO.Language = request.Value;
                    break;
                case CommonSettingsEnum.EnableReportLabel:
                    Boolean.TryParse(request.Value, out bool enableReportLabel);
                    srcUserInfoDTO.EnableReportLabel = enableReportLabel;
                    break;
                default:
                    return false;
            }
            var result = await _userDBService.UpdateUserAsync(srcUserInfoDTO.UserCode, srcUserInfoDTO.MappingTo<UserPasswordDTO>(), string.Empty, true);
            return result;
        }

        /// <summary>
        /// 查找日程数据请求
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<ClientScheduleDTO>> FindSchedulesAsync(FindSchedulesRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (request.StartTime < new DateTime(2000, 1, 1) || request.EndTime < new DateTime(2000, 1, 1))
            {
                ThrowCustomerException(CustomerRpcCode.ExamTimeIsEmpty, "Time is empty");
            }
            var schedules = await _userDBService.GetScheduleDBAsync(new GetScheduleDBRequest()
            {
                UserCode = userCode,
                StartTime = request.StartTime,
                EndTime = request.EndTime
            });
            return schedules?.MappingTo<List<ClientScheduleDTO>>();
        }

        private bool GetGradingProtectionConfigCache(string code)
        {
            var item = CacheMaintenance.Instance.Get<IGradingProtectionConfigManager>().Get(code);
            if (item != null)
            {
                if (item.Value.ToLower() == "true")
                {
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// 查询消息列表
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<PageResult<MessageExtendInfoDTO>> GetMessagesPageAsync(QueryMessageListRequest request)
        {
            var clientId = await GetClientIdByTokenAsync(request.Token);
            var notificationTypes = new List<NotificationTypeEnum>();
            notificationTypes = new List<NotificationTypeEnum>{
                NotificationTypeEnum.ApplyConsultationNotification,
                NotificationTypeEnum.RejectApplyConsultationNotification,
                NotificationTypeEnum.InviteeRejectApplyConsultationNotification,
                NotificationTypeEnum.InviteeConsultationNotification
            };
            if (request.TransactionType == TransactionTypeEnum.Consultion)
            {
                notificationTypes = new List<NotificationTypeEnum>{
                    NotificationTypeEnum.ApplyConsultationNotification,
                    NotificationTypeEnum.RejectApplyConsultationNotification,
                    NotificationTypeEnum.InviteeRejectApplyConsultationNotification,
                    NotificationTypeEnum.InviteeConsultationNotification
                };
            }
            var req = new GetMessagesDBPageRequest()
            {
                ClientId = clientId,
                Status = QueryCMSMessageStatusEnum.All,
                Keyword = request.Keyword,
                StartTime = DateTime.UtcNow.AddYears(-1),
                EndTime = DateTime.UtcNow,
                NotificationTypes = notificationTypes,
                PageSize = request.PageSize,
                CurrentPage = request.PageIndex
            };
            var pageResult = await _messageDBService.GetMessagesPageDBAsync(req);
            var result = new PageResult<MessageExtendInfoDTO>();
            if (pageResult != null && pageResult.PageData != null && pageResult.PageData.Count > 0)
            {
                result.PageIndex = pageResult.PageIndex;
                result.PageSize = pageResult.PageSize;
                result.TotalCount = pageResult.TotalCount;
                var list = new List<MessageExtendInfoDTO>();
                foreach (var dataItem in pageResult.PageData)
                {
                    var clientInfoDTO = dataItem.ClientInfos.FirstOrDefault(c => c.ClientId == clientId) ?? new ClientInfoDTO();
                    var messageExtendInfo = new MessageExtendInfoDTO()
                    {
                        MessageCode = dataItem.Code,
                        NotificationType = dataItem.NotificationType,
                        Content = dataItem.Content,
                        NotifyTime = dataItem.NotifyTime,
                        ReceiverType = dataItem.ReceiverType,
                        RelevanceCode = dataItem.RelevanceCode,
                        IsReaded = clientInfoDTO.IsReaded,
                        DeliveryTime = clientInfoDTO.DeliveryTime,
                        ReadTime = clientInfoDTO.ReadTime,
                        CreateTime = dataItem.CreateTime,
                        UpdateTime = dataItem.UpdateTime
                    };
                    list.Add(messageExtendInfo);
                }
                result.PageData = list.OrderByDescending(o => o.NotifyTime.Ticks + (o.IsReaded ? 0 : 999999999)).ToList();
            }
            return result;
        }

        /// <summary>
        /// 消息送达回调接口
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<bool> SetMessageDeliveryAsync(SetMessageDeliveryRequest request)
        {
            if (string.IsNullOrEmpty(request.MessageCode))
            {
                ThrowCustomerException(CustomerRpcCode.MessageCodeIsEmpty, "MessageCode is empty");
            }
            var clientId = await GetClientIdByTokenAsync(request.Token);
            var req = new UpdateMessagesDBRequest()
            {
                ClientId = clientId,
                MessageCode = request.MessageCode,
                IsReaded = request.IsReaded
            };
            var result = await _messageDBService.UpdateMessageInfoDBAsync(req);
            return result;
        }

        /// <summary>
        /// 设置消息为已读
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<bool> BatchSetMessageStatusAsync(SetMessageInfoReqeust request)
        {
            if (request.MessageCodes == null || request.MessageCodes.Count <= 0)
            {
                ThrowCustomerException(CustomerRpcCode.MessageCodesIsEmpty, "MessageCodes is empty");
            }
            var clientId = await GetClientIdByTokenAsync(request.Token);
            var req = new UpdateMessagesStatusDBequest()
            {
                ClientId = clientId,
                MessageCodes = request.MessageCodes,
            };
            var result = await _messageDBService.UpdateMessagesStatusDBAsync(req);
            return result;
        }

        public async Task<long> GetActiveUsersCount()
        {
            return await _userDBService.GetActiveUsersCount();
        }

        /// <summary>
        /// 获取公告内容列表接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<AnnouncementExtendInfoDTO>> QueryAnnouncementsByPageAsync(QueryAnnouncementListRequest request)
        {
            if (string.IsNullOrEmpty(request.Language))
            {
                request.Language = "en-US";
            }
            var req = new QueryAnnouncementsDBRequest()
            {
                AnnouncementStatus = AnnouncementStatusEnum.Released,
                AnnouncementType = request.AnnouncementType,
                Language = request.Language,
                Keyword = request.Keyword,
                EndTime = request.EndTime,
                StartTime = request.StartTime,
                PageSize = request.PageSize,
                CurrentPage = request.PageIndex
            };
            var pageResult = await _announcementDBService.QueryAnnouncementsDBByPageAsync(req);
            var result = new PageResult<AnnouncementExtendInfoDTO>();
            if (pageResult != null && pageResult.PageData != null && pageResult.PageData.Count > 0)
            {
                result.PageIndex = pageResult.PageIndex;
                result.PageSize = pageResult.PageSize;
                result.TotalCount = pageResult.TotalCount;
                var list = new List<AnnouncementExtendInfoDTO>();
                foreach (var dataItem in pageResult.PageData)
                {
                    var languageConfig = dataItem.LanguageConfigs?.FirstOrDefault(c => c.Language == request.Language) ?? new AnnouncementLanguageConfigDTO();
                    var announcementExtendInfo = new AnnouncementExtendInfoDTO()
                    {
                        AnnouncementCode = dataItem.Code,
                        AnnouncementType = dataItem.AnnouncementType,
                        Language = languageConfig.Language,
                        Title = languageConfig.Title,
                        Content = languageConfig.Content,
                    };
                    list.Add(announcementExtendInfo);
                }
                result.PageData = list;
            }
            return result;
        }

        /// <summary>
        /// 根据公告Code获取内容接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<AnnouncementExtendInfoDTO> GetAnnouncementInfoAsync(GetAnnouncementRequest request)
        {
            if (string.IsNullOrEmpty(request.AnnouncementCode))
            {
                ThrowCustomerException(CustomerRpcCode.AnnouncementCodeIsEmpty, "Announcement code is empty");
            }
            if (string.IsNullOrEmpty(request.Language))
            {
                ThrowCustomerException(CustomerRpcCode.LanguageCodeIsEmpty, "Language code is empty");
            }
            var req = new GetAnnouncementDBRequest()
            {
                AnnouncementCode = request.AnnouncementCode
            };
            var findEntity = await _announcementDBService.GetAnnouncementDBAsync(req);
            var result = new AnnouncementExtendInfoDTO();
            if (findEntity != null && !string.IsNullOrEmpty(findEntity.Code))
            {
                var languageConfig = findEntity.LanguageConfigs?.FirstOrDefault(c => c.Language == request.Language) ?? new AnnouncementLanguageConfigDTO();
                result = new AnnouncementExtendInfoDTO()
                {
                    AnnouncementCode = findEntity.Code,
                    AnnouncementType = findEntity.AnnouncementType,
                    Language = languageConfig.Language,
                    Title = languageConfig.Title,
                    Content = languageConfig.Content
                };
            }
            return result;
        }

        /// <summary>
        /// 获取未读消息数量
        /// </summary>
        /// <returns>未读消息编码列表</returns>
        public async Task<NoReadMessagesDTO> GetNoReadMessagesAsync(GetNoReadMessagesRequest request)
        {
            var clientId = await GetClientIdByTokenAsync(request.Token);
            var notificationTypes = new List<NotificationTypeEnum>();
            {
                notificationTypes = new List<NotificationTypeEnum>{
                    NotificationTypeEnum.ApplyConsultationNotification,
                    NotificationTypeEnum.RejectApplyConsultationNotification,
                    NotificationTypeEnum.InviteeRejectApplyConsultationNotification,
                    NotificationTypeEnum.InviteeConsultationNotification
                };
                if (request.TransactionType == TransactionTypeEnum.Consultion)
                {
                    notificationTypes = new List<NotificationTypeEnum>{
                        NotificationTypeEnum.ApplyConsultationNotification,
                        NotificationTypeEnum.RejectApplyConsultationNotification,
                        NotificationTypeEnum.InviteeRejectApplyConsultationNotification,
                        NotificationTypeEnum.InviteeConsultationNotification
                    };
                }
            }
            var dbRequest = new GetNoReadMessagesDBRequest();
            dbRequest.ClientId = clientId;
            dbRequest.NotificationTypes = notificationTypes;
            var result = await _messageDBService.GetNoReadMessagesDBAsync(dbRequest);
            var dto = new NoReadMessagesDTO
            {
                Count = result == null ? 0 : result.Count,
                NoReadCodes = result
            };
            return dto;
        }

        /// <summary>
        /// 删除单个ws连接
        /// </summary>
        /// <param name="request">登录token信息</param>
        /// <returns>是否成功</returns>
        public async Task<bool> RemoveUserSingleTokenAsync(RemoveUserSingleTokenRequest request)
        {
            var wsToken = request.WSToken;
            //移除websocket
            var removeRequest = new TokenRequest { Token = wsToken };
            var removeWebSocketResult = await _notificationService.RemoveSingleTokenAsync(removeRequest);
            return removeWebSocketResult;
        }

        #region 数据迁移
        /// <summary>
        /// 批量新增用户数据
        /// </summary>
        /// <param name="request">登录token信息</param>
        /// <returns>是否成功</returns>
        public async Task<bool> BatchInsertUserDataAsync(BatchInsertUserRequest request)
        {
            var res = await _userDBService.BatchInsertUserDataDBAsync(request);
            return res;
        }

        /// <summary>
        /// 清洗有医院的用户数据
        /// </summary>
        /// <param name="request">登录token信息</param>
        /// <returns>是否成功</returns>
        public async Task<bool> BatchUpdateUserOrganizationAsync(BatchInsertUserRequest request)
        {
            var res = await _userDBService.BatchUpdateUserOrganizationDBAsync(request);
            return res;
        }

        /// <summary>
        /// 获取待迁移时间
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<MigrateRecordDTO> QueryMigrateTimeAsync(QueryMigrateTimeRequest request)
        {
            var res = await _userDBService.QueryMigrateTimeDBAsync(request.IsInitMigreate);
            return res;
        }
        #endregion
    }
}
